==> dart_icalendar-1.0.1.tar.gz: lib/models/recurrence_model.dart <==
// ignore_for_file: constant_identifier_names

import 'dart:convert';

import 'package:collection/collection.dart';
import 'package:intl/intl.dart';

class RecurrenceRule {
  final Frequency frequency;
  final int interval;
  final int? count;
  final DateTime? until;
  final List<int>? bySecond;
  final List<int>? byMinute;
  final List<int>? byHour;
  final List<DayOfWeek?>? byDay;
  final List<int>? byMonthDay;
  final List<int>? byYearDay;
  final List<int>? byWeekNo;
  final List<int>? byMonth;
  final List<int>? bySetPosition;
  final Weekday? weekStart;

  RecurrenceRule({
    required this.frequency,
    this.interval = 1,
    this.count,
    this.until,
    this.bySecond,
    this.byMinute,
    this.byHour,
    this.byDay,
    this.byMonthDay,
    this.byYearDay,
    this.byWeekNo,
    this.byMonth,
    this.bySetPosition,
    this.weekStart,
  });

  String toICalString() {
    final buffer = StringBuffer();
    buffer.write('RRULE:FREQ=${frequency.name}');
    if (weekStart != null) {
      buffer.write(';WKST=${weekStart!.toICalString()}');
    }
    if (interval > 1) {
      buffer.write(';INTERVAL=$interval');
    }
    if (count != null) {
      buffer.write(';COUNT=$count');
    }

    if (until != null) {
      buffer.write(
          ';UNTIL=${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(until!.toUtc())}');
    }
    if (bySecond != null && bySecond!.isNotEmpty) {
      buffer.write(';BYSECOND=${bySecond!.join(',')}');
    }
    if (byMinute != null && byMinute!.isNotEmpty) {
      buffer.write(';BYMINUTE=${byMinute!.join(',')}');
    }
    if (byHour != null && byHour!.isNotEmpty) {
      buffer.write(';BYHOUR=${byHour!.join(',')}');
    }
    if (byDay != null && byDay!.isNotEmpty) {
      buffer.write(
          ';BYDAY=${byDay!.map((day) => day?.toICalString()).join(',')}');
    }
    if (byMonthDay != null && byMonthDay!.isNotEmpty) {
      buffer.write(';BYMONTHDAY=${byMonthDay!.join(',')}');
    }
    if (byYearDay != null && byYearDay!.isNotEmpty) {
      buffer.write(';BYYEARDAY=${byYearDay!.join(',')}');
    }
    if (byWeekNo != null && byWeekNo!.isNotEmpty) {
      buffer.write(';BYWEEKNO=${byWeekNo!.join(',')}');
    }
    if (byMonth != null && byMonth!.isNotEmpty) {
      buffer.write(';BYMONTH=${byMonth!.join(',')}');
    }
    if (bySetPosition != null && bySetPosition!.isNotEmpty) {
      buffer.write(';BYSETPOS=${bySetPosition!.join(',')}');
    }

    return buffer.toString();
  }

  factory RecurrenceRule.fromICalString(String icalString) {
    final frequencyString =
        icalString.replaceAll('RRULE:FREQ=', '').split(';').first.toUpperCase();
    final frequency = Frequency.values.firstWhere(
        (f) => f.name.toUpperCase() == frequencyString,
        orElse: () => Frequency.SECONDLY);
    final interval = icalString.contains(';INTERVAL=')
        ? int.parse(icalString.split(';INTERVAL=').last.split(';').first)
        : 1;
    final count = icalString.contains(';COUNT=')
        ? int.parse(icalString.split(';COUNT=').last.split(';').first)
        : null;
    final untilString = icalString.contains(';UNTIL=')
        ? icalString.split(';UNTIL=').last
        : null;
    final wkstString = icalString.contains(';WKST=')
        ? icalString.split(';WKST=').last.split(';').first
        : null;
    final wkst =
        wkstString != null ? WeekdayExtension.fromICalString(wkstString) : null;
    DateTime? until;
    if (untilString != null) {
      final regex = RegExp(r'^\d{8}T\d{6}Z');
      final dateString = regex.stringMatch(untilString)?.trim();
      until = dateString != null ? DateTime.parse(dateString).toUtc() : null;
    }
    final bySecond = _parseIntListFromIcalString(icalString, 'BYSECOND');
    final byMinute = _parseIntListFromIcalString(icalString, 'BYMINUTE');
    final byHour = _parseIntListFromIcalString(icalString, 'BYHOUR');
    var days = icalString.contains(';BYDAY=')
        ? icalString
            .split(';BYDAY=')
            .last
            .split(',')
            .map((e) => e.trim())
            .toList()
        : null;

    List<DayOfWeek>? byDay;
    if (days != null) {
      for (var i = 0; i < days.length; i++) {
        byDay ??= [];
        final day = days[i];
        byDay.add(DayOfWeek.fromICalString(day));
      }
    } else {
      byDay = null;
    }

    final byMonthDay = _parseIntListFromIcalString(icalString, 'BYMONTHDAY');
    final byYearDay = _parseIntListFromIcalString(icalString, 'BYYEARDAY');
    final byWeekNo = _parseIntListFromIcalString(icalString, 'BYWEEKNO');
    final byMonth = _parseIntListFromIcalString(icalString, 'BYMONTH');
    final bySetPosition = _parseIntListFromIcalString(icalString, 'BYSETPOS');

    return RecurrenceRule(
      frequency: frequency,
      interval: interval,
      count: count,
      until: until,
      bySecond: bySecond,
      byMinute: byMinute,
      byHour: byHour,
      byDay: byDay,
      byMonthDay: byMonthDay,
      byYearDay: byYearDay,
      byWeekNo: byWeekNo,
      byMonth: byMonth,
      weekStart: wkst,
      bySetPosition: bySetPosition,
    );
  }

  static List<int>? _parseIntListFromIcalString(
      String icalString, String parameterName) {
    return icalString.contains(';$parameterName=')
        ? icalString
            .split(';$parameterName=')
            .last
            .split(',')
            .map(int.parse)
            .toList()
        : null;
  }

  RecurrenceRule copyWith({
    Frequency? frequency,
    int? interval,
    int? count,
    DateTime? until,
    List<int>? bySecond,
    List<int>? byMinute,
    List<int>? byHour,
    List<DayOfWeek>? byDay,
    List<int>? byMonthDay,
    List<int>? byYearDay,
    List<int>? byWeekNo,
    List<int>? byMonth,
    List<int>? bySetPosition,
  }) {
    return RecurrenceRule(
      frequency: frequency ?? this.frequency,
      interval: interval ?? this.interval,
      count: count ?? this.count,
      until: until ?? this.until,
      bySecond: bySecond ?? this.bySecond,
      byMinute: byMinute ?? this.byMinute,
      byHour: byHour ?? this.byHour,
      byDay: byDay ?? this.byDay,
      byMonthDay: byMonthDay ?? this.byMonthDay,
      byYearDay: byYearDay ?? this.byYearDay,
      byWeekNo: byWeekNo ?? this.byWeekNo,
      byMonth: byMonth ?? this.byMonth,
      bySetPosition: bySetPosition ?? this.bySetPosition,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'frequency': frequency.index,
      'interval': interval,
      'count': count,
      'until': until?.millisecondsSinceEpoch,
      'bySecond': bySecond,
      'byMinute': byMinute,
      'byHour': byHour,
      'byDay': byDay?.map((x) => x?.toMap()).toList(),
      'byMonthDay': byMonthDay,
      'byYearDay': byYearDay,
      'byWeekNo': byWeekNo,
      'byMonth': byMonth,
      'bySetPosition': bySetPosition,
    };
  }

  factory RecurrenceRule.fromMap(Map<String, dynamic> map) {
    return RecurrenceRule(
      frequency: Frequency.values[map['frequency'] ?? 0],
      interval: map['interval']?.toInt() ?? 0,
      count: map['count']?.toInt(),
      until: map['until'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['until'])
          : null,
      bySecond: List<int>.from(map['bySecond']),
      byMinute: List<int>.from(map['byMinute']),
      byHour: List<int>.from(map['byHour']),
      byDay: map['byDay'] != null
          ? List<DayOfWeek>.from(map['byDay']?.map((x) => DayOfWeek.fromMap(x)))
          : null,
      byMonthDay: List<int>.from(map['byMonthDay']),
      byYearDay: List<int>.from(map['byYearDay']),
      byWeekNo: List<int>.from(map['byWeekNo']),
      byMonth: List<int>.from(map['byMonth']),
      bySetPosition: List<int>.from(map['bySetPosition']),
    );
  }

  String toJson() => json.encode(toMap());

  factory RecurrenceRule.fromJson(String source) =>
      RecurrenceRule.fromMap(json.decode(source));

  @override
  String toString() {
    return 'RecurrenceRule(frequency: $frequency, interval: $interval, count: $count, until: $until, bySecond: $bySecond, byMinute: $byMinute, byHour: $byHour, byDay: $byDay, byMonthDay: $byMonthDay, byYearDay: $byYearDay, byWeekNo: $byWeekNo, byMonth: $byMonth, bySetPosition: $bySetPosition)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is RecurrenceRule &&
        other.frequency == frequency &&
        other.interval == interval &&
        other.count == count &&
        other.until == until &&
        listEquals(other.bySecond, bySecond) &&
        listEquals(other.byMinute, byMinute) &&
        listEquals(other.byHour, byHour) &&
        listEquals(other.byDay, byDay) &&
        listEquals(other.byMonthDay, byMonthDay) &&
        listEquals(other.byYearDay, byYearDay) &&
        listEquals(other.byWeekNo, byWeekNo) &&
        listEquals(other.byMonth, byMonth) &&
        listEquals(other.bySetPosition, bySetPosition);
  }

  @override
  int get hashCode {
    return frequency.hashCode ^
        interval.hashCode ^
        count.hashCode ^
        until.hashCode ^
        bySecond.hashCode ^
        byMinute.hashCode ^
        byHour.hashCode ^
        byDay.hashCode ^
        byMonthDay.hashCode ^
        byYearDay.hashCode ^
        byWeekNo.hashCode ^
        byMonth.hashCode ^
        bySetPosition.hashCode;
  }
}

enum Frequency {
  SECONDLY,
  MINUTELY,
  HOURLY,
  DAILY,
  WEEKLY,
  MONTHLY,
  YEARLY,
}

extension FrequencyExtension on Frequency {
  String get name => toString().split('.').last;
}

class DayOfWeek {
  final Weekday weekday;
  final int? weekNumber;

  DayOfWeek({required this.weekday, this.weekNumber});

  String toICalString() {
    final buffer = StringBuffer();
    if (weekNumber != null) {
      buffer.write('$weekNumber');
    }
    buffer.write(weekday.toICalString());
    return buffer.toString();
  }

  factory DayOfWeek.fromICalString(String dayString) {
    final weekNumberString = dayString.substring(0, dayString.length - 2);
    final weekNumber =
        weekNumberString.isNotEmpty ? int.parse(weekNumberString) : null;
    final weekdayString = dayString.substring(dayString.length - 2);
    final weekday = WeekdayExtension.fromICalString(weekdayString);
    if (weekday == null) {
      throw ArgumentError('Invalid day of week: $dayString');
    }
    return DayOfWeek(weekday: weekday, weekNumber: weekNumber);
  }

  Map<String, dynamic> toMap() {
    return {
      'weekday': weekday.index,
      'weekNumber': weekNumber,
    };
  }

  factory DayOfWeek.fromMap(Map<String, dynamic> map) {
    return DayOfWeek(
      weekday: Weekday.values[map['weekday']],
      weekNumber: map['weekNumber']?.toInt(),
    );
  }

  String toJson() => json.encode(toMap());

  factory DayOfWeek.fromJson(String source) =>
      DayOfWeek.fromMap(json.decode(source));
}

enum Weekday {
  MO,
  TU,
  WE,
  TH,
  FR,
  SA,
  SU,
}

extension WeekdayExtension on Weekday {
  String toICalString() {
    switch (this) {
      case Weekday.MO:
        return 'MO';
      case Weekday.TU:
        return 'TU';
      case Weekday.WE:
        return 'WE';
      case Weekday.TH:
        return 'TH';
      case Weekday.FR:
        return 'FR';
      case Weekday.SA:
        return 'SA';
      case Weekday.SU:
        return 'SU';
    }
  }

  static Weekday? fromICalString(String dayString) {
    switch (dayString) {
      case 'MO':
        return Weekday.MO;
      case 'TU':
        return Weekday.TU;
      case 'WE':
        return Weekday.WE;
      case 'TH':
        return Weekday.TH;
      case 'FR':
        return Weekday.FR;
      case 'SA':
        return Weekday.SA;
      case 'SU':
        return Weekday.SU;
      default:
        return null;
    }
  }
}

==> dart_icalendar-1.0.2.tar.gz: lib/models/recurrence_model.dart <==
// ignore_for_file: constant_identifier_names

import 'dart:convert';

import 'package:collection/collection.dart';
import 'package:intl/intl.dart';

class RecurrenceRule {
  final Frequency frequency;
  final int interval;
  final int? count;
  final DateTime? until;
  final List<int>? bySecond;
  final List<int>? byMinute;
  final List<int>? byHour;
  final List<DayOfWeek?>? byDay;
  final List<int>? byMonthDay;
  final List<int>? byYearDay;
  final List<int>? byWeekNo;
  final List<int>? byMonth;
  final List<int>? bySetPosition;
  final Weekday? weekStart;

  RecurrenceRule({
    required this.frequency,
    this.interval = 1,
    this.count,
    this.until,
    this.bySecond,
    this.byMinute,
    this.byHour,
    this.byDay,
    this.byMonthDay,
    this.byYearDay,
    this.byWeekNo,
    this.byMonth,
    this.bySetPosition,
    this.weekStart,
  });

  String toICalString() {
    final buffer = StringBuffer();
    buffer.write('RRULE:FREQ=${frequency.name}');
    if (weekStart != null) {
      buffer.write(';WKST=${weekStart!.toICalString()}');
    }
    if (interval > 1) {
      buffer.write(';INTERVAL=$interval');
    }
    if (count != null) {
      buffer.write(';COUNT=$count');
    }

    if (until != null) {
      buffer.write(
          ';UNTIL=${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(until!.toUtc())}');
    }
    if (bySecond != null && bySecond!.isNotEmpty) {
      buffer.write(';BYSECOND=${bySecond!.join(',')}');
    }
    if (byMinute != null && byMinute!.isNotEmpty) {
      buffer.write(';BYMINUTE=${byMinute!.join(',')}');
    }
    if (byHour != null && byHour!.isNotEmpty) {
      buffer.write(';BYHOUR=${byHour!.join(',')}');
    }
    if (byDay != null && byDay!.isNotEmpty) {
      buffer.write(
          ';BYDAY=${byDay!.map((day) => day?.toICalString()).join(',')}');
    }
    if (byMonthDay != null && byMonthDay!.isNotEmpty) {
      buffer.write(';BYMONTHDAY=${byMonthDay!.join(',')}');
    }
    if (byYearDay != null && byYearDay!.isNotEmpty) {
      buffer.write(';BYYEARDAY=${byYearDay!.join(',')}');
    }
    if (byWeekNo != null && byWeekNo!.isNotEmpty) {
      buffer.write(';BYWEEKNO=${byWeekNo!.join(',')}');
    }
    if (byMonth != null && byMonth!.isNotEmpty) {
      buffer.write(';BYMONTH=${byMonth!.join(',')}');
    }
    if (bySetPosition != null && bySetPosition!.isNotEmpty) {
      buffer.write(';BYSETPOS=${bySetPosition!.join(',')}');
    }

    return buffer.toString();
  }

  factory RecurrenceRule.fromICalString(String icalString) {
    final frequencyString =
        icalString.replaceAll('RRULE:FREQ=', '').split(';').first.toUpperCase();
    final frequency = Frequency.values.firstWhere(
        (f) => f.name.toUpperCase() == frequencyString,
        orElse: () => Frequency.SECONDLY);
    final interval = icalString.contains(';INTERVAL=')
        ? int.parse(icalString.split(';INTERVAL=').last.split(';').first)
        : 1;
    final count = icalString.contains(';COUNT=')
        ? int.parse(icalString.split(';COUNT=').last.split(';').first)
        : null;
    final untilString = icalString.contains(';UNTIL=')
        ? icalString.split(';UNTIL=').last
        : null;
    final wkstString = icalString.contains(';WKST=')
        ? icalString.split(';WKST=').last.split(';').first
        : null;
    final wkst =
        wkstString != null ? WeekdayExtension.fromICalString(wkstString) : null;
    DateTime? until;
    if (untilString != null) {
      final regex = RegExp(r'^\d{8}T\d{6}Z');
      final dateString = regex.stringMatch(untilString)?.trim();
      until = dateString != null ? DateTime.parse(dateString).toUtc() : null;
    }
    final bySecond = _parseIntListFromIcalString(icalString, 'BYSECOND');
    final byMinute = _parseIntListFromIcalString(icalString, 'BYMINUTE');
    final byHour = _parseIntListFromIcalString(icalString, 'BYHOUR');
    var days = icalString.contains(';BYDAY=')
        ? icalString
            .split(';BYDAY=')
            .last
            .split(',')
            .map((e) => e.trim())
            .toList()
        : null;

    List<DayOfWeek>? byDay;
    if (days != null) {
      for (var i = 0; i < days.length; i++) {
        byDay ??= [];
        final day = days[i];
        byDay.add(DayOfWeek.fromICalString(day));
      }
    } else {
      byDay = null;
    }

    final byMonthDay = _parseIntListFromIcalString(icalString, 'BYMONTHDAY');
    final byYearDay = _parseIntListFromIcalString(icalString, 'BYYEARDAY');
    final byWeekNo = _parseIntListFromIcalString(icalString, 'BYWEEKNO');
    final byMonth = _parseIntListFromIcalString(icalString, 'BYMONTH');
    final bySetPosition = _parseIntListFromIcalString(icalString, 'BYSETPOS');

    return RecurrenceRule(
      frequency: frequency,
      interval: interval,
      count: count,
      until: until,
      bySecond: bySecond,
      byMinute: byMinute,
      byHour: byHour,
      byDay: byDay,
      byMonthDay: byMonthDay,
      byYearDay: byYearDay,
      byWeekNo: byWeekNo,
      byMonth: byMonth,
      weekStart: wkst,
      bySetPosition: bySetPosition,
    );
  }

  static List<int>? _parseIntListFromIcalString(
      String icalString, String parameterName) {
    return icalString.contains(';$parameterName=')
        ? icalString
            .split(';$parameterName=')
            .last
            .split(',')
            .map(int.parse)
            .toList()
        : null;
  }

  RecurrenceRule copyWith({
    Frequency? frequency,
    int? interval,
    int? count,
    DateTime? until,
    List<int>? bySecond,
    List<int>? byMinute,
    List<int>? byHour,
    List<DayOfWeek>? byDay,
    List<int>? byMonthDay,
    List<int>? byYearDay,
    List<int>? byWeekNo,
    List<int>? byMonth,
    List<int>? bySetPosition,
  }) {
    return RecurrenceRule(
      frequency: frequency ?? this.frequency,
      interval: interval ?? this.interval,
      count: count ?? this.count,
      until: until ?? this.until,
      bySecond: bySecond ?? this.bySecond,
      byMinute: byMinute ?? this.byMinute,
      byHour: byHour ?? this.byHour,
      byDay: byDay ?? this.byDay,
      byMonthDay: byMonthDay ?? this.byMonthDay,
      byYearDay: byYearDay ?? this.byYearDay,
      byWeekNo: byWeekNo ?? this.byWeekNo,
      byMonth: byMonth ?? this.byMonth,
      bySetPosition: bySetPosition ?? this.bySetPosition,
    );
  }

  @override
  String toString() {
    return 'RecurrenceRule(frequency: $frequency, interval: $interval, count: $count, until: $until, bySecond: $bySecond, byMinute: $byMinute, byHour: $byHour, byDay: $byDay, byMonthDay: $byMonthDay, byYearDay: $byYearDay, byWeekNo: $byWeekNo, byMonth: $byMonth, bySetPosition: $bySetPosition)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is RecurrenceRule &&
        other.frequency == frequency &&
        other.interval == interval &&
        other.count == count &&
        other.until == until &&
        listEquals(other.bySecond, bySecond) &&
        listEquals(other.byMinute, byMinute) &&
        listEquals(other.byHour, byHour) &&
        listEquals(other.byDay, byDay) &&
        listEquals(other.byMonthDay, byMonthDay) &&
        listEquals(other.byYearDay, byYearDay) &&
        listEquals(other.byWeekNo, byWeekNo) &&
        listEquals(other.byMonth, byMonth) &&
        listEquals(other.bySetPosition, bySetPosition);
  }

  @override
  int get hashCode {
    return frequency.hashCode ^
        interval.hashCode ^
        count.hashCode ^
        until.hashCode ^
        bySecond.hashCode ^
        byMinute.hashCode ^
        byHour.hashCode ^
        byDay.hashCode ^
        byMonthDay.hashCode ^
        byYearDay.hashCode ^
        byWeekNo.hashCode ^
        byMonth.hashCode ^
        bySetPosition.hashCode;
  }

  Map<String, dynamic> toMap() {
    return {
      'frequency': frequency.index,
      'interval': interval,
      'count': count,
      'until': until?.millisecondsSinceEpoch,
      'by_second': bySecond,
      'by_minute': byMinute,
      'by_hour': byHour,
      'by_day': byDay?.map((x) => x?.toMap()).toList(),
      'by_month_day': byMonthDay,
      'by_year_day': byYearDay,
      'by_week_no': byWeekNo,
      'by_month': byMonth,
      'by_set_position': bySetPosition,
      'week_start': weekStart?.index,
    };
  }

  factory RecurrenceRule.fromMap(Map<String, dynamic> map) {
    return RecurrenceRule(
      frequency: Frequency.values[map['frequency']],
      interval: map['interval']?.toInt() ?? 0,
      count: map['count']?.toInt(),
      until: map['until'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['until'])
          : null,
      bySecond: List<int>.from(map['by_second']),
      byMinute: List<int>.from(map['by_minute']),
      byHour: List<int>.from(map['by_hour']),
      byDay: map['by_day'] != null
          ? List<DayOfWeek?>.from(
              map['byDay']?.map((x) => DayOfWeek?.fromMap(x)))
          : null,
      byMonthDay: List<int>.from(map['by_month_day']),
      byYearDay: List<int>.from(map['by_year_day']),
      byWeekNo: List<int>.from(map['by_week_no']),
      byMonth: List<int>.from(map['by_month']),
      bySetPosition: List<int>.from(map['by_set_position']),
      weekStart:
          map['week_start'] != null ? Weekday.values[map['weekStart']] : null,
    );
  }

  String toJson() => json.encode(toMap());

  factory RecurrenceRule.fromJson(String source) =>
      RecurrenceRule.fromMap(json.decode(source));
}

enum Frequency {
  SECONDLY,
  MINUTELY,
  HOURLY,
  DAILY,
  WEEKLY,
  MONTHLY,
  YEARLY,
}

extension FrequencyExtension on Frequency {
  String get name => toString().split('.').last;
}

class DayOfWeek {
  final Weekday weekday;
  final int? weekNumber;

  DayOfWeek({required this.weekday, this.weekNumber});

  String toICalString() {
    final buffer = StringBuffer();
    if (weekNumber != null) {
      buffer.write('$weekNumber');
    }
    buffer.write(weekday.toICalString());
    return buffer.toString();
  }

  factory DayOfWeek.fromICalString(String dayString) {
    final weekNumberString = dayString.substring(0, dayString.length - 2);
    final weekNumber =
        weekNumberString.isNotEmpty ? int.parse(weekNumberString) : null;
    final weekdayString = dayString.substring(dayString.length - 2);
    final weekday = WeekdayExtension.fromICalString(weekdayString);
    if (weekday == null) {
      throw ArgumentError('Invalid day of week: $dayString');
    }
    return DayOfWeek(weekday: weekday, weekNumber: weekNumber);
  }

  Map<String, dynamic> toMap() {
    return {
      'weekday': weekday.index,
      'week_number': weekNumber,
    };
  }

  factory DayOfWeek.fromMap(Map<String, dynamic> map) {
    return DayOfWeek(
      weekday: Weekday.values[map['weekday']],
      weekNumber: map['week_number']?.toInt(),
    );
  }

  String toJson() => json.encode(toMap());

  factory DayOfWeek.fromJson(String source) =>
      DayOfWeek.fromMap(json.decode(source));
}

enum Weekday {
  MO,
  TU,
  WE,
  TH,
  FR,
  SA,
  SU,
}

extension WeekdayExtension on Weekday {
  String toICalString() {
    switch (this) {
      case Weekday.MO:
        return 'MO';
      case Weekday.TU:
        return 'TU';
      case Weekday.WE:
        return 'WE';
      case Weekday.TH:
        return 'TH';
      case Weekday.FR:
        return 'FR';
      case Weekday.SA:
        return 'SA';
      case Weekday.SU:
        return 'SU';
    }
  }

  static Weekday? fromICalString(String dayString) {
    switch (dayString) {
      case 'MO':
        return Weekday.MO;
      case 'TU':
        return Weekday.TU;
      case 'WE':
        return Weekday.WE;
      case 'TH':
        return Weekday.TH;
      case 'FR':
        return Weekday.FR;
      case 'SA':
        return Weekday.SA;
      case 'SU':
        return Weekday.SU;
      default:
        return null;
    }
  }
}
